CREATE SCHEMA people_scoreboard;

CREATE TABLE people_scoreboard.people
(
    id              SERIAL PRIMARY KEY,
    parent1Id       INT REFERENCES people_scoreboard.people (id),
    parent2Id       INT REFERENCES people_scoreboard.people (id),
    birth_timestamp TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP
);

CREATE TABLE people_scoreboard.activities
(
    id    SERIAL PRIMARY KEY,
    name  VARCHAR(255),
    value INT
);

CREATE TABLE people_scoreboard.activity_log
(
    id              SERIAL PRIMARY KEY,
    activity_id     INT REFERENCES people_scoreboard.activities (id),
    human_id        INT REFERENCES people_scoreboard.people (id),
    start_timestamp TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP,
    end_timestamp   TIMESTAMPTZ
);

CREATE TABLE people_scoreboard.occupations
(
    id    SERIAL PRIMARY KEY,
    name  VARCHAR(255),
    value INT
);

CREATE TABLE people_scoreboard.occupation_log
(
    id              SERIAL PRIMARY KEY,
    occupation_id   INT REFERENCES people_scoreboard.occupations (id),
    human_id        INT REFERENCES people_scoreboard.people (id),
    start_timestamp TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP,
    end_timestamp   TIMESTAMPTZ,
    grade           INT
);

CREATE OR REPLACE FUNCTION people_scoreboard.get_activity_value(_humanId INT)
    RETURNS INT AS
$$
DECLARE
    _totalValue INT;
BEGIN
    SELECT SUM(value)
    INTO _totalValue
    FROM people_scoreboard.activity_log
             JOIN people_scoreboard.activities
                  ON people_scoreboard.activity_log.activity_id = people_scoreboard.activities.id
    WHERE people_scoreboard.activity_log.human_id = _humanId;

    RETURN _totalValue;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION people_scoreboard.get_occupation_value(_humanId INT)
    RETURNS INT AS
$$
DECLARE
    _totalValue INT;
BEGIN
    SELECT SUM(value * grade * EXTRACT(YEAR FROM AGE(COALESCE(end_timestamp, CURRENT_TIMESTAMP), start_timestamp)))
    INTO _totalValue
    FROM people_scoreboard.occupation_log
             JOIN people_scoreboard.occupations
                  ON people_scoreboard.occupation_log.occupation_id = people_scoreboard.occupations.id
    WHERE people_scoreboard.occupation_log.human_id = _humanId;

    RETURN _totalValue;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION people_scoreboard.get_human_value(_humanId INT)
    RETURNS INT AS
$$
DECLARE
    _activityValue   INT;
    _occupationValue INT;
BEGIN
    _activityValue := people_scoreboard.get_activity_value(_humanId);
    _occupationValue := people_scoreboard.get_occupation_value(_humanId);

    RETURN _activityValue + _occupationValue;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION people_scoreboard.get_all_people_values(_offset INT, _limit INT)
    RETURNS TABLE
            (
                humanId INT,
                value   INT
            )
AS
$$
BEGIN
    RETURN QUERY
        SELECT id, people_scoreboard.get_human_value(id)
        FROM people_scoreboard.people
        ORDER BY id
        LIMIT _limit OFFSET _offset;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE VIEW people_scoreboard.human_logs AS
SELECT p.id                                                             AS humanId,
       json_agg(json_build_object('name', a.name, 'value', a.value, 'start_timestamp',
                                  al.start_timestamp, 'end_timestamp',
                                  al.end_timestamp))                    AS activity_log,
       json_agg(json_build_object('name', o.name, 'value', o.value, 'start_timestamp',
                                  ol.start_timestamp, 'end_timestamp',
                                  ol.end_timestamp, 'grade', ol.grade)) AS occupation_log
FROM people_scoreboard.people p
         LEFT JOIN people_scoreboard.activity_log al ON p.id = al.human_id
         LEFT JOIN people_scoreboard.occupation_log ol ON p.id = ol.human_id
         LEFT JOIN people_scoreboard.activities a ON al.activity_id = a.id
         LEFT JOIN people_scoreboard.occupations o ON ol.occupation_id = o.id
GROUP BY p.id;




